<?php
/**
 * CSRF Component
 *
 */
App::import('Core', array('String', 'Security'));

/**
 * CsrfComponent
 */
class CsrfComponent extends Object {

/**
 * The controller method that will be called if this request is black-hole'd
 *
 * @var string
 * @access public
 */
	public $blackHoleCallback = 'csrfError';


/**
 * Whether to validate POST data.  Set to false to disable for data coming from 3rd party
 * services, etc.
 *
 * @var boolean
 * @access public
 */
	public $validatePost = true;

/**
 * Other components used by the Security component
 *
 * @var array
 * @access public
 */
	public $components = array('RequestHandler', 'Session');

/**
 * CSRF実行無視フラグ
 *
 * @var boolean trueの場合は実行されないため、sessionのtokenが新しい値に更新されない
 * @access public
 */
	public $ignore = false;

/**
 * Initialize the SecurityComponent
 *
 * @param object $controller Controller instance for the request
 * @param array $settings Settings to set to the component
 * @return void
 * @access public
 */
	public function initialize(&$controller, $settings = array()) {
		$this->_set($settings);
	}

/**
 * Component startup. All security checking happens here.
 *
 * @param object $controller Instantiating controller
 * @return void
 * @access public
 */
	public function startup(&$controller) {
		if ($this->ignore) {
			return;
		}

		$isPost = ($this->RequestHandler->isPost() || $this->RequestHandler->isPut());
		if ($isPost && $this->validatePost) {
			if ($this->_validatePost($controller) === false) {
				$this->Session->setFlash(__("message.csrf.validate", true), 'message');
				if (!$this->blackHole($controller, 'auth')) {
					return null;
				}
			}
		}
		$this->_generateToken($controller);
	}


/**
 * Black-hole an invalid request with a 404 error or custom callback. If SecurityComponent::$blackHoleCallback
 * is specified, it will use this callback by executing the method indicated in $error
 *
 * @param object $controller Instantiating controller
 * @param string $error Error method
 * @return mixed If specified, controller blackHoleCallback's response, or no return otherwise
 * @access public
 * @see SecurityComponent::$blackHoleCallback
 * @link http://book.cakephp.org/view/1307/blackHole-object-controller-string-error
 */
	public function blackHole(&$controller, $error = '') {
		if ($this->blackHoleCallback == null) {
			$code = 404;
			$controller->redirect(null, $code, true);
		} else {
			return $this->_callback($controller, $this->blackHoleCallback, array($error));
		}
	}


/**
 * Validate submitted form
 *
 * @param object $controller Instantiating controller
 * @return bool true if submitted form is valid
 * @access protected
 */
	protected function _validatePost(&$controller) {
		if (empty($controller->data)) {
			return true;
		}
		$data = $controller->data;

		if (!isset($data['_Token']['key'])) {
			return false;
		}
		$token = $data['_Token']['key'];

		if (!$this->Session->check('_Token')) {
			return false;
		}
		$session_token = $this->Session->read('_Token');
		return $token === $session_token;
	}

/**
 * Add authentication key for new form posts
 *
 * @param object $controller Instantiating controller
 * @return bool Success
 * @access protected
 */
	protected function _generateToken(&$controller) {
		$authKey = Security::generateAuthKey();
		$controller->params['_Token'] = array('key' => $authKey);
		$this->Session->write('_Token', $authKey);
		return true;
	}

	public function clear() {
		$this->Session->delete('_Token');
	}

/**
 * Calls a controller callback method
 *
 * @param object $controller Controller to run callback on
 * @param string $method Method to execute
 * @param array $params Parameters to send to method
 * @return mixed Controller callback method's response
 * @access protected
 */
	protected function _callback(&$controller, $method, $params = array()) {
		if (is_callable(array($controller, $method))) {
			return call_user_func_array(array(&$controller, $method), empty($params) ? null : $params);
		} else {
			return null;
		}
	}
}
