<?php
/**
 * Tag Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */

class Tag extends AppModel{
	var $name = 'Tag';
	var $validate = array(
		'name' => array(
			'minlength' => array(
				'rule' => array('minLength', '1'),
				'required' => true,
				'message' => 'validation.enter',
			),
			'maxlength' => array(
				'rule' => array('maxLength', '100'),
				'message' => 'validation.maxlength',
				'last' => true
			),
		),
	);

	/**
	 * Top画面に表示するタグを返す。
	 *
	 * @param $accountid
	 * @return array
	 */
	function getUserTags($accountid) {
		return $this->find('all', array(
				'fields' => array('group_id', 'name', 'public'),
				'conditions' => array(
					'OR' => array(
						'account_id' => $accountid,
						'public' => 1,
					)
				)
			)
		);
	}

	/**
	 * タグ追加時に表示するタグ名の候補を返す。
	 *
	 * @param $accountid
	 * @return array
	 */
	function getCandidateTagNames($accountid) {
		$tagNames = $this->find('list', array(
				'fields' => array('name'),
				'conditions' => array(
					'OR' => array(
						'account_id' => $accountid,
						'public' => 1,
					)
				)
			)
		);
		return array_unique($tagNames);
	}

	/**
	 * タグ名のリストを返す。
	 * 引数に指定した値をconditionsに指定してfindを実行する。
	 *
	 * @param $accountid nullable
	 * @param $groupid nullable
	 * @param $public 0 or 1 or null
	 * @return array
	 */
	private function getTagNames($accountid, $groupid, $public) {
		$conditions = array();
		if (!is_null($accountid)) {
			$conditions['account_id'] = $accountid;
		}
		if (!is_null($groupid)) {
			$conditions['group_id'] = $groupid;
		}
		if (!is_null($public)) {
			$conditions['public'] = $public;
		}
		$tagNames = $this->find('list',
			array('fields' => array('name'),
			      'conditions' => $conditions));
		return array_unique($tagNames);
	}

	/**
	 * ユーザタグを返す。
	 *
	 * @param $accountid nullable
	 * @param $groupid nullable
	 * @return array
	 */
	function getUserTagNames($accountid, $groupid=null) {
		return $this->getTagNames($accountid, $groupid, 0);
	}

	/**
	 * 公開タグを返す。
	 *
	 * @param $accountid nullable
	 * @param $groupid nullable
	 * @return array
	 */
	function getPublicTagNames($accountid, $groupid=null) {
		return $this->getTagNames($accountid, $groupid, 1);
	}

	/**
	 * タグを更新(追加/削除)する。
	 * アカウントIDとSPコネクタID(グループID)に紐づくユーザタグ(または公開タグ)を
	 * 引数$newTagNamesの内容に置き換える($newTagNamesに含まれないタグは削除する)。
	 * Transactionは、呼び出し元で設定すること。
	 *
	 * @param $accountid
	 * @param $groupid
	 * @param $public 0 or 1
	 * @paran $newTagNames array
	 * @return bool 成功した場合はtrueを返す。失敗した場合はfalseを返す。
	 */
	private function updateTags($accountid, $groupid, $public, $newTagNames) {
		if ($public) {
			$curTagNames = $this->getTagNames(null, $groupid, $public);
		} else {
			$curTagNames = $this->getTagNames($accountid, $groupid, $public);
		}
		if (!empty($curTagNames)) {
			if (empty($newTagNames)) {
				$delTagNames = $curTagNames;
			} else {
				$delTagNames = array_diff($curTagNames, $newTagNames);
			}
			if (!empty($delTagNames)) {
				$result = $this->deleteAll(array('id' => array_keys($delTagNames)));
				if (!$result) {
					return false;
				}
			}
		}
		if (!empty($newTagNames)) {
			if (empty($curTagNames)) {
				$addTagNames = array_values($newTagNames);
			} else {
				$addTagNames = array_values(array_diff($newTagNames, $curTagNames));
			}
			if (!empty($addTagNames)) {
				$newTags = array_fill_keys(array_keys($addTagNames),
					array('account_id' => $accountid,
					      'group_id' => $groupid,
					      'public' => $public));
				foreach($addTagNames as $i => $tagName) {
					$newTags[$i]['name'] = $tagName;
				}
				$results = $this->saveAll($newTags, array('atomic' => false));
				if (!$this->checkSaveAll($results)) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * タグを更新(追加/削除)する。
	 * アカウントIDとSPコネクタID(グループID)に紐づくユーザタグを
	 * 引数$newTagNamesの内容に置き換える($newTagNamesに含まれないタグは削除する)。
	 * Transactionは、呼び出し元で設定すること。
	 *
	 * @param $accountid
	 * @param $groupid
	 * @paran $newTagNames array
	 * @return bool 成功した場合はtrueを返す。失敗した場合はfalseを返す。
	 */
	function updateUserTags($accountid, $groupid, $newTagNames) {
		return $this->updateTags($accountid, $groupid, 0, $newTagNames);
	}

	/**
	 * タグを更新(追加/削除)する。
	 * アカウントIDとSPコネクタID(グループID)に紐づく公開タグを
	 * 引数$newTagNamesの内容に置き換える($newTagNamesに含まれないタグは削除する)。
	 * Transactionは、呼び出し元で設定すること。
	 *
	 * @param $accountid
	 * @param $groupid
	 * @paran $newTagNames array
	 * @return bool 成功した場合はtrueを返す。失敗した場合はfalseを返す。
	 */
	function updatePublicTags($accountid, $groupid, $newTagNames) {
		return $this->updateTags($accountid, $groupid, 1, $newTagNames);
	}
}
